# Import
from apm import *
    
# Select server
s = 'http://byu.apmonitor.com'

a = 'reactor'

# Clear previous application
apm(s,a,'clear all')

# Load model file
apm_load(s,a,'cstr.apm')

# Load time points for future predictions
csv_load(s,a,'cstr.csv')

# APM Variable Classification
# class = FV, MV, SV, CV
#   F or FV = Fixed value - parameter may change to a new value every cycle
#   M or MV = Manipulated variable - independent variable over time horizon
#   S or SV = State variable - model variable for viewing
#   C or CV = Controlled variable - model variable for control

#Parameters
FVs = 'Cooling_Temp','Flow','Feed_Conc','Feed_Temp'
MVs = '---'

#Variables
SVs = 'Concentration','---'
CVs = 'Temperature','---'

# Set up variable classifications for data flow
for x in FVs: apm_info(s,a,'FV',x)
for x in MVs: apm_info(s,a,'MV',x)
for x in SVs: apm_info(s,a,'SV',x)
for x in CVs: apm_info(s,a,'CV',x)

# Options

# turn on historization to see past results in web-viewer
apm_option(s,a,'nlc.hist_hor',100)

# set web plot update frequency
apm_option(s,a,'nlc.web_plot_freq',3)

# set number of nodes for collocation
apm_option(s,a,'nlc.nodes',6)

# imode (1=ss, 2=mpu, 3=rto, 4=sim, 5=mhe, 6=nlc)

# initialize with steady state
apm_option(s,a,'nlc.imode',1)
solver_output = apm(s,a,'solve')

# retrieve steady state cooling temperature
Tc = apm_tag(s,a,'cooling_temp.newval')

# swith to dynamic simulation
apm_option(s,a,'nlc.imode',4)

time = 0.0
for isim in range(101):

    # Step changes in cooling temperature
    if (isim==5):
        Tc = 300.0
    elif (isim==50):
        Tc = 280.0
    elif (isim==80):
        Tc = 260.0

    # Insert Cooling temperature
    apm_meas(s,a,'cooling_temp',Tc)

    # Run on server
    solver_output = apm(s,a,'solve')

    # Read reactor temperature and concentration
    T = apm_tag(s,a,'temperature.model')
    Ca = apm_tag(s,a,'concentration.model')

    # Print data for import into Excel
    print('{0:2f} {1:2f} {2:2f} {3:2f}'.format(time,Tc,T,Ca))

    # Increment time
    time = time + 0.05

    if (isim==1):
        # Open Web Viewer and Display Link
        url = apm_web(s,a)
