# Import
from apm import *
    
# Select server
s = 'http://byu.apmonitor.com'

a = 'reactor'

# Clear previous application
apm(s,a,'clear all')

# Load model file
apm_load(s,a,'cstr.apm')

# Load time points for future predictions
csv_load(s,a,'cstr.csv')

# APM Variable Classification
# class = FV, MV, SV, CV
#   F or FV = Fixed value - parameter may change to a new value every cycle
#   M or MV = Manipulated variable - independent variable over time horizon
#   S or SV = State variable - model variable for viewing
#   C or CV = Controlled variable - model variable for control

#Parameters
FVs = 'Tsp','Cooling_Temp','Flow','Feed_Conc','Feed_Temp'
MVs = '---'

#Variables
SVs = 'Concentration','---'
CVs = 'Temperature','---'

# Set up variable classifications for data flow
for x in FVs: apm_info(s,a,'FV',x)
for x in MVs: apm_info(s,a,'MV',x)
for x in SVs: apm_info(s,a,'SV',x)
for x in CVs: apm_info(s,a,'CV',x)

# Options

# turn on historization to see past results in web-viewer
apm_option(s,a,'nlc.hist_hor',100)

# set web plot update frequency
apm_option(s,a,'nlc.web_plot_freq',3)

# set number of nodes for collocation
apm_option(s,a,'nlc.nodes',6)

# imode (1=ss, 2=mpu, 3=rto, 4=sim, 5=mhe, 6=nlc)

# initialize with steady state
apm_option(s,a,'nlc.imode',1)
solver_output = apm(s,a,'solve')

# retrieve steady state cooling temperature and reactor temperature
Tc = apm_tag(s,a,'cooling_temp.newval')
T = apm_tag(s,a,'temperature.model')

# swith to dynamic simulation
apm_option(s,a,'nlc.imode',4)

# set point
Tsp = 310.0

# Tc upper and lower limits
Tc_upper = 350.0
Tc_lower = 250.0

# initialize values
time = 0.0
dt = 0.05
int_error = 0.0
Tc0 = Tc

# Basic tuning rules
# P = 1/Kp = 1.0
P = 1.0
# I = P/taup = 1.0 / 0.57
I = 2.0

for isim in range(101):

    # PID Controller ######################
    # Change temperature set point
    if (isim==50):
        Tsp = 290.0

    # Calculate temperature setpoint error
    error = Tsp - T

    # Update Integral approximation
    int_error = int_error + error * dt

    # PI Controller
    Tc = Tc0 + P * error + I * int_error

    # Check for clipping and integral windup
    if (Tc > Tc_upper) or (Tc < Tc_lower):
        Tc = min(Tc,Tc_upper)
        Tc = max(Tc,Tc_lower)
        # anti-reset windup
        int_error = int_error - error * dt
    # #####################################

    # Insert Cooling temperature and setpoint
    apm_meas(s,a,'cooling_temp',Tc)
    apm_meas(s,a,'Tsp',Tsp)

    # Run on server
    solver_output = apm(s,a,'solve')

    # Read reactor temperature and concentration
    T = apm_tag(s,a,'temperature.model')
    Ca = apm_tag(s,a,'concentration.model')

    # Print data for import into Excel
    print('{0:2f} {1:2f} {2:2f} {3:2f}'.format(time,Tc,T,Ca))

    # Increment time
    time = time + dt

    if (isim==1):
        # Open Web Viewer and Display Link
        url = apm_web(s,a)
